//  This file is part of foo_ir2.
//
//  foo_ir2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  foo_ir2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with foo_ir2. If not, see <http://www.gnu.org/licenses/>.

#include "resource.h"
#include "foo_ir2.h"

using namespace pfc;

foo_ir2::foo_ir2()
{
	// we're creating a message-only window for handling serial port events
	WNDCLASSEX wnd;

	wnd.cbSize = sizeof( WNDCLASSEX );
	wnd.style = CS_HREDRAW | CS_VREDRAW;
	wnd.lpfnWndProc = WndProc;
	wnd.cbClsExtra = wnd.cbWndExtra = 0;
	wnd.hInstance = core_api::get_my_instance();
	wnd.hIcon = wnd.hIconSm = LoadIcon(NULL, IDI_APPLICATION);
	wnd.hCursor = LoadCursor(NULL, IDC_ARROW);
	wnd.hbrBackground = (HBRUSH) COLOR_WINDOW;;
	wnd.lpszMenuName = NULL;
	wnd.lpszClassName = _T("ir2wndclass");

	if( !RegisterClassEx( &wnd ) )
	{
		char err[255];
		sprintf_s( err, 255, "foo_ir2: can't register wndclass (%d)", GetLastError() );
		console::error( err );
	}

	g_hMessageWnd = CreateWindowEx(
		WS_EX_CLIENTEDGE,
		_T("ir2wndclass"),
		_T("ir2wnd"),
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		240, 120,
		HWND_MESSAGE, // message-only window
		NULL,
		core_api::get_my_instance(),
		NULL );

	if( g_hMessageWnd == NULL )
	{
		char err[255];
		sprintf_s( err, 255, "foo_ir2: can't create wnd (%d)", GetLastError() );
		console::error( err );
	}

	if( cfg_ir2_enabled ) OpenSerialPort();
}

foo_ir2::~foo_ir2()
{
	CloseSerialPort();
}

// windowproc for our hidden message window
LRESULT CALLBACK foo_ir2::WndProc( HWND hWnd, UINT uMsg, WPARAM wp, LPARAM lp )
{
	switch( uMsg )
	{
	case WU_SERIALPORTEVENT:
		CSerial::EEvent eEvent = CSerial::EEvent( LOWORD(wp) );
		CSerial::EError eError = CSerial::EError( HIWORD(wp) );

		switch( eEvent )
		{
			case CSerial::EEventRecv:
				g_debug( "serial event received" );
				DWORD dwBytesRead;
				char in;
				static_api_ptr_t<play_control> pc;

				do
				{
					g_SerialPort.Read( (void*)&in, 1, &dwBytesRead );
					if( dwBytesRead == 0 )
						continue;
					g_SerialQueue.push( in );

					if( g_eLearning != nothing || ( GetTickCount() - g_dwTimeOfLastCommand <= (DWORD)cfg_delay.get_value() ) )
						continue;

					if( !g_fOKReceived && memcmp( g_SerialQueue.getdata(), "OK", 2 ) == 0 )
					{
						g_fOKReceived = true;
						console::info( "foo_ir2: OK received" );
						g_SerialQueue.erase();
						break;
					}

					// commands
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().play_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: play command received" );
						if (pc->is_paused())
							pc->toggle_pause();
						else
							pc->start();
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = GetTickCount();
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().stop_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: stop command received" );
						pc->stop();
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = GetTickCount();
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().next_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: next track command received" );
						pc->play_start( play_control::track_command_next );
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = GetTickCount();
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().nextrand_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: next random track command received" );
						pc->play_start( play_control::track_command_rand );
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = GetTickCount();
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().prev_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: previous track command received" );
						pc->play_start( play_control::track_command_prev );
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = GetTickCount();
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().backw_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: seek backward command received" );
						pc->playback_seek_delta( -5 );
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = 0;
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().forw_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: seek forward command received" );
						pc->playback_seek_delta( 5 );
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = 0;
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().volup_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: volume up command received" );
						pc->volume_up();
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = 0;
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().voldown_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: volume down command received" );
						pc->volume_down();
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = 0;
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().pause_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: pause command received" );
						pc->toggle_pause();
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = GetTickCount();
						break;
					}
					if( memcmp( g_SerialQueue.getdata(), cfg_ircodes.get_value().mute_code, cfg_codelength.get_value() ) == 0 )
					{
						console::info( "foo_ir2: mute command received" );
						pc->volume_mute_toggle();
						g_SerialQueue.erase();
						g_dwTimeOfLastCommand = GetTickCount();
						break;
					}
				}
				while( dwBytesRead > 0 );

				// if learning mode is on and we got a full-length code
				if( g_eLearning != nothing && g_SerialQueue.length() == cfg_codelength.get_value() )
				{
					t_ircodes irc;
					memcpy( &irc, &cfg_ircodes.get_value(), sizeof(irc) );
					g_dwTimeOfLastCommand = GetTickCount();

					switch( g_eLearning )
					{
					case play:
						memcpy( irc.play_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_PLAY ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case pause:
						memcpy( irc.pause_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_PAUSE ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case stop:
						memcpy( irc.stop_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_STOP ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case next:
						memcpy( irc.next_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_NEXT ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case nextrandom:
						memcpy( irc.nextrand_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_NEXTRANDOM ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case previous:
						memcpy( irc.prev_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_PREVIOUS ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case backward:
						memcpy( irc.backw_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_BACKWARD ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case forward:
						memcpy( irc.forw_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_FORWARD ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case volumeup:
						memcpy( irc.volup_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_VOLUMEUP ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case volumedown:
						memcpy( irc.voldown_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_VOLUMEDOWN ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					case mute:
						memcpy( irc.mute_code, g_SerialQueue.getdata(), cfg_codelength.get_value() );
						SendMessage( GetDlgItem( g_hPrefWnd, IDC_MUTE ), BM_SETCHECK, BST_UNCHECKED, 0 );
						break;
					}
					cfg_ircodes = irc;
					SendMessage( g_hPrefWnd, WU_UPDATEWINDOW, 0, 0 );
				}
				break;
		}
		return 0;
	}

	return DefWindowProc( hWnd, uMsg, wp, lp );
}

void foo_ir2::OpenSerialPort()
{
	LPCTSTR port = AnsiToUnicode( cfg_comport.get_ptr() );
	LONG lLastError;
	bool fErrorHappened = false;
	WCHAR err[255];

	CloseSerialPort();

	StringCbPrintf( err, 255, _T("foo_ir2: opening port %s"), port );
	LPSTR errstr = UnicodeToAnsi( err );
	console::info( errstr );
	SAFE_DELETE( errstr );

	lLastError = g_SerialPort.Open( port, g_hMessageWnd, WU_SERIALPORTEVENT );
	if ( lLastError != ERROR_SUCCESS )
	{
		StringCbPrintf( err, 255, _T("foo_ir2: can't open port %s"), port );
		LPSTR errstr = UnicodeToAnsi( err );
		console::error( errstr );
		SAFE_DELETE( errstr );

		fErrorHappened = true;
	}

	lLastError = g_SerialPort.Setup( (CSerial::EBaudrate)cfg_baudrate.get_value(), (CSerial::EDataBits)cfg_databits.get_value(), (CSerial::EParity)cfg_parity.get_value(), (CSerial::EStopBits)cfg_stopbits.get_value() );
	if ( lLastError != ERROR_SUCCESS )
	{
		StringCbPrintf( err, 255, _T("foo_ir2: can't modify port %s settings"), port );
		LPSTR errstr = UnicodeToAnsi( err );
		console::error( errstr );
		SAFE_DELETE( errstr );

		fErrorHappened = true;
	}
	lLastError = g_SerialPort.SetupHandshaking( (CSerial::EHandshake)cfg_flowcontrol.get_value() );
	if ( lLastError != ERROR_SUCCESS )
	{
		StringCbPrintf( err, 255, _T("foo_ir2: can't modify port %s handshake setting"), port );
		LPSTR errstr = UnicodeToAnsi( err );
		console::error( errstr );
		SAFE_DELETE( errstr );

		fErrorHappened = true;
	}

	if( cfg_setdtr )
		g_SerialPort.SetDTR();
	if( cfg_setrts )
		g_SerialPort.SetRTS();

	g_SerialQueue.setsize( cfg_codelength.get_value() );
	g_SerialQueue.erase();

	// waiting for the IR device to power on
	Sleep( cfg_startupdelay.get_value() );

	g_fOKReceived = false; // for IR receivers which send back OK string after initialization
	g_dwTimeOfLastCommand = 0;

	// sending initialization string if needed
	if( cfg_initstring.length() > 0 )
	{
		DWORD w;
		g_SerialPort.Write( cfg_initstring.get_ptr(), cfg_initstring.length(), &w );
		if( w != cfg_initstring.length() )
		{
			console::error( "foo_ir2: error while sending initialization string" );
			fErrorHappened = true;
		}
	}

	if( !fErrorHappened )
	{
		StringCbPrintf( err, 255, _T("foo_ir2: %s opened successfully"), port );
		LPSTR errstr = UnicodeToAnsi( err );
		console::info( errstr );
		SAFE_DELETE( errstr );
	}
	SAFE_DELETE( port );
}

void foo_ir2::CloseSerialPort()
{
	g_SerialPort.Close();
}

/////////////////////////////
// PREFERENCES PAGE WINDOW //
/////////////////////////////
void ir2_config::UpdateWindow()
{
	if( cfg_ir2_enabled )
	{
		SendMessage( GetDlgItem( g_hPrefWnd, IDC_ENABLED ), BM_SETCHECK, BST_CHECKED, 0 );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_PORT ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_BAUDRATE ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_DATABITS ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_PARITY ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_STOPBITS ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_FLOWCONTROL ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_INITSTRING ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_STARTUPDELAY ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_DELAY ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_CODELENGTH ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_DTRHIGH ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_RTSHIGH ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_LEARNGROUP ), true );

		if( g_eLearning != nothing ) // learning mode was on
				console::info( "foo_ir2: learning mode off" );
		g_eLearning = nothing;
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_PLAY ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = play;
			console::info( "foo_ir2: learning play button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_STOP ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = stop;
			console::info( "foo_ir2: learning stop button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_PAUSE ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = pause;
			console::info( "foo_ir2: learning pause button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_NEXT ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = next;
			console::info( "foo_ir2: learning next track button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_NEXTRANDOM ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = nextrandom;
			console::info( "foo_ir2: learning next random track button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_PREVIOUS ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = previous;
			console::info( "foo_ir2: learning previous track button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_BACKWARD ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = backward;
			console::info( "foo_ir2: learning backward seek button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_FORWARD ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = forward;
			console::info( "foo_ir2: learning forward seek button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_VOLUMEUP ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = volumeup;
			console::info( "foo_ir2: learning volume up button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_VOLUMEDOWN ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = volumedown;
			console::info( "foo_ir2: learning volume down button..." );
		}
		if( SendMessage( GetDlgItem( g_hPrefWnd, IDC_MUTE ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		{
			g_eLearning = mute;
			console::info( "foo_ir2: learning mute button..." );
		}
		if( g_eLearning != nothing )
			g_SerialQueue.erase();
		LearnCheck();
	}
	else
	{
		SendMessage( GetDlgItem( g_hPrefWnd, IDC_ENABLED ), BM_SETCHECK, BST_UNCHECKED, 0 );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_PORT ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_BAUDRATE ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_DATABITS ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_PARITY ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_STOPBITS ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_FLOWCONTROL ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_INITSTRING ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_STARTUPDELAY ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_DELAY ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_COMBO_CODELENGTH ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_DTRHIGH ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_RTSHIGH ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_LEARNGROUP ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_PLAY ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_PAUSE ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_STOP ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_NEXT ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_NEXTRANDOM ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_PREVIOUS ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_BACKWARD ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_FORWARD ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_MUTE ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_VOLUMEUP ), false );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_VOLUMEDOWN ), false );
	}
}

void ir2_config::LearnCheck()
{
	if( g_eLearning != nothing )
	{
		// disabling the button we're not learning
		if( g_eLearning != play ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_PLAY ), false );
		if( g_eLearning != pause ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_PAUSE ), false );
		if( g_eLearning != stop ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_STOP ), false );
		if( g_eLearning != next) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_NEXT ), false );
		if( g_eLearning != nextrandom ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_NEXTRANDOM ), false );
		if( g_eLearning != previous ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_PREVIOUS ), false );
		if( g_eLearning != backward ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_BACKWARD ), false );
		if( g_eLearning != forward ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_FORWARD ), false );
		if( g_eLearning != mute ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_MUTE ), false );
		if( g_eLearning != volumeup ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_VOLUMEUP ), false );
		if( g_eLearning != volumedown ) EnableWindow( GetDlgItem( g_hPrefWnd, IDC_VOLUMEDOWN ), false );
	}
	else
	{
		// enabling all the buttons
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_PLAY ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_PAUSE ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_STOP ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_NEXT ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_NEXTRANDOM ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_PREVIOUS ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_BACKWARD ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_FORWARD ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_MUTE ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_VOLUMEUP ), true );
		EnableWindow( GetDlgItem( g_hPrefWnd, IDC_VOLUMEDOWN ), true );
	}
}

BOOL CALLBACK ir2_config::ProcWindowMsgs( HWND hWnd, UINT uMsg, WPARAM wp, LPARAM lp )
{
	switch( uMsg )
	{
	case WM_SHOWWINDOW:
	case WU_UPDATEWINDOW:
		UpdateWindow();
		break;

	case WM_INITDIALOG:
		if( CSerial::CheckPort( _T("COM1") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM1") );
		if( CSerial::CheckPort( _T("COM2") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM2") );
		if( CSerial::CheckPort( _T("COM3") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM3") );
		if( CSerial::CheckPort( _T("COM4") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM4") );
		if( CSerial::CheckPort( _T("COM5") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM5") );
		if( CSerial::CheckPort( _T("COM6") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM6") );
		if( CSerial::CheckPort( _T("COM7") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM7") );
		if( CSerial::CheckPort( _T("COM8") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM8") );
		if( CSerial::CheckPort( _T("COM9") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM9") );
		if( CSerial::CheckPort( _T("COM10") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("COM10") );
		if( CSerial::CheckPort( _T("\\\\.\\CNCA0") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("CNCA0") );
		if( CSerial::CheckPort( _T("\\\\.\\CNCB0") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("CNCB0") );
		if( CSerial::CheckPort( _T("\\\\.\\CNCA1") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("CNCA1") );
		if( CSerial::CheckPort( _T("\\\\.\\CNCB1") ) == CSerial::EPortAvailable )
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)_T("CNCB1") );
		if( g_SerialPort.IsOpen() )
		{
			WCHAR* tmp = AnsiToUnicode( cfg_comport.get_ptr() );
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_ADDSTRING, 0, (LPARAM)tmp );
			SAFE_DELETE( tmp );
		}

		LPCTSTR port;
		port = AnsiToUnicode( cfg_comport.get_ptr() );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_SETCURSEL, SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_FINDSTRING, 0, (LPARAM)port ), 0 );
		SAFE_DELETE( port );

		SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_ADDSTRING, 0, (LPARAM)_T("1200") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_ADDSTRING, 0, (LPARAM)_T("2400") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_ADDSTRING, 0, (LPARAM)_T("4800") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_ADDSTRING, 0, (LPARAM)_T("9600") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_ADDSTRING, 0, (LPARAM)_T("14400") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_ADDSTRING, 0, (LPARAM)_T("19200") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_ADDSTRING, 0, (LPARAM)_T("38400") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_ADDSTRING, 0, (LPARAM)_T("57600") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_ADDSTRING, 0, (LPARAM)_T("115200") );
		switch( cfg_baudrate )
		{
		case CSerial::EBaud1200:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_SETCURSEL, 0, 0 );
			break;
		case CSerial::EBaud2400:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_SETCURSEL, 1, 0 );
			break;
		case CSerial::EBaud4800:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_SETCURSEL, 2, 0 );
			break;
		default:
		case CSerial::EBaud9600:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_SETCURSEL, 3, 0 );
			break;
		case CSerial::EBaud14400:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_SETCURSEL, 4, 0 );
			break;
		case CSerial::EBaud19200:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_SETCURSEL, 5, 0 );
			break;
		case CSerial::EBaud38400:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_SETCURSEL, 6, 0 );
			break;
		case CSerial::EBaud57600:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_SETCURSEL, 7, 0 );
			break;
		case CSerial::EBaud115200:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_BAUDRATE ), CB_SETCURSEL, 8, 0 );
			break;
		}

		SendMessage( GetDlgItem( hWnd, IDC_COMBO_DATABITS ), CB_ADDSTRING, 0, (LPARAM)_T("5") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_DATABITS ), CB_ADDSTRING, 0, (LPARAM)_T("6") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_DATABITS ), CB_ADDSTRING, 0, (LPARAM)_T("7") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_DATABITS ), CB_ADDSTRING, 0, (LPARAM)_T("8") );
		switch( cfg_databits )
		{
		case CSerial::EData5:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_DATABITS ), CB_SETCURSEL, 0, 0 );
			break;
		case CSerial::EData6:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_DATABITS ), CB_SETCURSEL, 1, 0 );
			break;
		case CSerial::EData7:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_DATABITS ), CB_SETCURSEL, 2, 0 );
			break;
		default:
		case CSerial::EData8:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_DATABITS ), CB_SETCURSEL, 3, 0 );
			break;
		}

		SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_ADDSTRING, 0, (LPARAM)_T("None") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_ADDSTRING, 0, (LPARAM)_T("Even") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_ADDSTRING, 0, (LPARAM)_T("Odd") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_ADDSTRING, 0, (LPARAM)_T("Mark") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_ADDSTRING, 0, (LPARAM)_T("Space") );
		switch( cfg_parity )
		{
		default:
		case CSerial::EParNone:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_SETCURSEL, 0, 0 );
			break;
		case CSerial::EParEven:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_SETCURSEL, 1, 0 );
			break;
		case CSerial::EParOdd:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_SETCURSEL, 2, 0 );
			break;
		case CSerial::EParMark:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_SETCURSEL, 3, 0 );
			break;
		case CSerial::EParSpace:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_SETCURSEL, 4, 0 );
			break;
		}

		SendMessage( GetDlgItem( hWnd, IDC_COMBO_STOPBITS ), CB_ADDSTRING, 0, (LPARAM)_T("1") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_STOPBITS ), CB_ADDSTRING, 0, (LPARAM)_T("1.5") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_STOPBITS ), CB_ADDSTRING, 0, (LPARAM)_T("2") );
		switch( cfg_stopbits )
		{
		default:
		case CSerial::EStop1:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_STOPBITS ), CB_SETCURSEL, 0, 0 );
			break;
		case CSerial::EStop1_5:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_STOPBITS ), CB_SETCURSEL, 1, 0 );
			break;
		case CSerial::EStop2:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_STOPBITS ), CB_SETCURSEL, 2, 0 );
			break;
		}

		SendMessage( GetDlgItem( hWnd, IDC_COMBO_FLOWCONTROL ), CB_ADDSTRING, 0, (LPARAM)_T("None") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_FLOWCONTROL ), CB_ADDSTRING, 0, (LPARAM)_T("Hardware") );
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_FLOWCONTROL ), CB_ADDSTRING, 0, (LPARAM)_T("Xon / Xoff") );
		switch( cfg_flowcontrol )
		{
		default:
		case CSerial::EHandshakeOff:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_FLOWCONTROL ), CB_SETCURSEL, 0, 0 );
			break;
		case CSerial::EHandshakeHardware:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_FLOWCONTROL ), CB_SETCURSEL, 1, 0 );
			break;
		case CSerial::EHandshakeSoftware:
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_FLOWCONTROL ), CB_SETCURSEL, 2, 0 );
			break;
		}

		WCHAR* initstring;
		initstring = AnsiToUnicode( cfg_initstring.get_ptr() );
		SetDlgItemText( hWnd, IDC_INITSTRING, initstring );
		SAFE_DELETE( initstring );

		WCHAR tmp[20];
		StringCbPrintf( tmp, 20, _T("%d"), cfg_startupdelay.get_value() );
		SetDlgItemText( hWnd, IDC_STARTUPDELAY, tmp );

		StringCbPrintf( tmp, 20, _T("%d"), cfg_delay.get_value() );
		SetDlgItemText( hWnd, IDC_DELAY, tmp );

		SendMessage( GetDlgItem( hWnd, IDC_COMBO_CODELENGTH ), CB_ADDSTRING, 0, (LPARAM)_T("1 byte") );
		for( int i=2; i <= MAXCODELENGTH; i++ )
		{
			StringCbPrintf( tmp, 20, _T("%d bytes"), i );
			SendMessage( GetDlgItem( hWnd, IDC_COMBO_CODELENGTH ), CB_ADDSTRING, 0, (LPARAM)tmp );
		}
		SendMessage( GetDlgItem( hWnd, IDC_COMBO_CODELENGTH ), CB_SETCURSEL, cfg_codelength.get_value()-1, 0 );

		if( cfg_setdtr )
			SendMessage( GetDlgItem( hWnd, IDC_DTRHIGH ), BM_SETCHECK, BST_CHECKED, 0 );
		if( cfg_setrts )
			SendMessage( GetDlgItem( hWnd, IDC_RTSHIGH ), BM_SETCHECK, BST_CHECKED, 0 );
		break;

	case WM_DESTROY:
		break;
	///////////////////////
	// USER INTERACTIONS //
	///////////////////////
	case WM_COMMAND:
		switch( LOWORD(wp) )
		{
		case IDC_ENABLED:
			if( SendMessage( GetDlgItem( hWnd, IDC_ENABLED ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
			{
				cfg_ir2_enabled = true;
				foo_ir2::OpenSerialPort();
			}	
			else
			{
				cfg_ir2_enabled = false;
				foo_ir2::CloseSerialPort();
			}
			UpdateWindow();
			break;
		case IDC_COMBO_PORT:
			if( HIWORD(wp) == CBN_SELCHANGE )
			{
				int cursel = SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_GETCURSEL, 0, 0 );
				size_t tmplen = SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_GETLBTEXTLEN, cursel, 0 );
				WCHAR* tmp = new WCHAR[tmplen+1];
				SendMessage( GetDlgItem( hWnd, IDC_COMBO_PORT ), CB_GETLBTEXT, cursel, (LPARAM)tmp );
				char* tmp2 = UnicodeToAnsi( tmp );
				cfg_comport = tmp2;
				SAFE_DELETE( tmp );
				SAFE_DELETE( tmp2 );
			}
			break;
		case IDC_COMBO_BITRATE:
			if( HIWORD(wp) == CBN_SELCHANGE )
			{
				int cursel = SendMessage( GetDlgItem( hWnd, IDC_COMBO_BITRATE ), CB_GETCURSEL, 0, 0 );
				switch( cursel )
				{
				case 0:	cfg_baudrate = CSerial::EBaud1200; break;
				case 1:	cfg_baudrate = CSerial::EBaud2400; break;
				case 2:	cfg_baudrate = CSerial::EBaud4800; break;
				default:
				case 3:	cfg_baudrate = CSerial::EBaud9600; break;
				case 4:	cfg_baudrate = CSerial::EBaud14400; break;
				case 5:	cfg_baudrate = CSerial::EBaud19200; break;
				case 6:	cfg_baudrate = CSerial::EBaud38400; break;
				case 7:	cfg_baudrate = CSerial::EBaud57600; break;
				case 8:	cfg_baudrate = CSerial::EBaud115200; break;
				}
			}
			break;
		case IDC_COMBO_DATABITS:
			if( HIWORD(wp) == CBN_SELCHANGE )
			{
				int cursel = SendMessage( GetDlgItem( hWnd, IDC_COMBO_DATABITS ), CB_GETCURSEL, 0, 0 );
				switch( cursel )
				{
				case 0:	cfg_databits = CSerial::EData5; break;
				case 1:	cfg_databits = CSerial::EData6; break;
				case 2:	cfg_databits = CSerial::EData7; break;
				default:
				case 3:	cfg_databits = CSerial::EData8; break;
				}
			}
			break;
		case IDC_COMBO_PARITY:
			if( HIWORD(wp) == CBN_SELCHANGE )
			{
				int cursel = SendMessage( GetDlgItem( hWnd, IDC_COMBO_PARITY ), CB_GETCURSEL, 0, 0 );
				switch( cursel )
				{
				default:
				case 0:	cfg_parity = CSerial::EParNone; break;
				case 1:	cfg_parity = CSerial::EParEven; break;
				case 2:	cfg_parity = CSerial::EParOdd; break;
				case 3:	cfg_parity = CSerial::EParMark; break;
				case 4:	cfg_parity = CSerial::EParSpace; break;
				}
			}
			break;
		case IDC_COMBO_STOPBITS:
			if( HIWORD(wp) == CBN_SELCHANGE )
			{
				int cursel = SendMessage( GetDlgItem( hWnd, IDC_COMBO_STOPBITS ), CB_GETCURSEL, 0, 0 );
				switch( cursel )
				{
				default:
				case 0:	cfg_stopbits = CSerial::EStop1; break;
				case 1:	cfg_stopbits = CSerial::EStop1_5; break;
				case 2:	cfg_stopbits = CSerial::EStop2; break;
				}
			}
			break;
		case IDC_COMBO_FLOWCONTROL:
			if( HIWORD(wp) == CBN_SELCHANGE )
			{
				int cursel = SendMessage( GetDlgItem( hWnd, IDC_COMBO_FLOWCONTROL ), CB_GETCURSEL, 0, 0 );
				switch( cursel )
				{
				default:
				case 0:	cfg_flowcontrol = CSerial::EHandshakeOff; break;
				case 1:	cfg_flowcontrol = CSerial::EHandshakeHardware; break;
				case 2:	cfg_flowcontrol = CSerial::EHandshakeSoftware; break;
				}
			}
			break;
		case IDC_COMBO_CODELENGTH:
			if( HIWORD(wp) == CBN_SELCHANGE )
				cfg_codelength = SendMessage( GetDlgItem( hWnd, IDC_COMBO_CODELENGTH ), CB_GETCURSEL, 0, 0 ) + 1;
			break;
		case IDC_DTRHIGH:
			if( SendMessage( GetDlgItem( hWnd, IDC_DTRHIGH ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
				cfg_setdtr = true;
			else
				cfg_setdtr = false;
			break;
		case IDC_RTSHIGH:
			if( SendMessage( GetDlgItem( hWnd, IDC_RTSHIGH ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
				cfg_setrts = true;
			else
				cfg_setrts = false;
			break;
		case IDC_INITSTRING:
			if( HIWORD(wp) == EN_CHANGE )
			{
				size_t len = SendMessage( GetDlgItem( hWnd, IDC_INITSTRING ), EM_LINELENGTH, 0, 0 );
				WCHAR* tmp = new WCHAR[len+1];
				tmp[0] = len;
				SendMessage( GetDlgItem( hWnd, IDC_INITSTRING ), EM_GETLINE, 0, (LPARAM)tmp );
				char* tmp2 = UnicodeToAnsi( tmp );
				cfg_initstring = tmp2;
				SAFE_DELETE( tmp );
				SAFE_DELETE( tmp2 );
			}
			break;
		case IDC_STARTUPDELAY:
			if( HIWORD(wp) == EN_CHANGE )
			{
				size_t len = SendMessage( GetDlgItem( hWnd, IDC_STARTUPDELAY ), EM_LINELENGTH, 0, 0 );
				WCHAR* tmp = new WCHAR[len+1];
				tmp[0] = len;
				SendMessage( GetDlgItem( hWnd, IDC_STARTUPDELAY ), EM_GETLINE, 0, (LPARAM)tmp );
				cfg_startupdelay = _wtoi(tmp);
				SAFE_DELETE( tmp );
			}
			break;
		case IDC_DELAY:
			if( HIWORD(wp) == EN_CHANGE )
			{
				size_t len = SendMessage( GetDlgItem( hWnd, IDC_DELAY ), EM_LINELENGTH, 0, 0 );
				WCHAR* tmp = new WCHAR[len+1];
				tmp[0] = len;
				SendMessage( GetDlgItem( hWnd, IDC_DELAY ), EM_GETLINE, 0, (LPARAM)tmp );
				cfg_delay = _wtoi(tmp);
				SAFE_DELETE( tmp );
			}
			break;
		////////////////////////////
		case IDC_PLAY:
		case IDC_STOP:
		case IDC_PAUSE:
		case IDC_NEXT:
		case IDC_NEXTRANDOM:
		case IDC_PREVIOUS:
		case IDC_BACKWARD:
		case IDC_FORWARD:
		case IDC_VOLUMEUP:
		case IDC_VOLUMEDOWN:
		case IDC_MUTE:
			UpdateWindow();
			break;
		}
	}

	return 0;
}
